require(raster)
require(data.table)
require(reshape2)
require(rgdal)

data_dir_root = "/Users/peteralexander/Documents/LURG/Model Comp/Phase1/LUC4C MIP"

boundaries <- raster(file.path(data_dir_root, 'country_boundaries.asc'))
crs(boundaries) <- '+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs'
eu27 = boundaries %in% fread(file.path(data_dir_root, 'eu27.txt'))$numCode
eu27 = extend(crop(eu27, extent(-15,35,30,75)), extent(-180,180,-90,90))
names(eu27) = "eu27"

################################################################################################################
### CLUMONDO
################################################################################################################

## EXTRACT GLOBAL VALUES

# see 'CLUMondoExtract.R' 

################################################################################################################
### IMAGE
################################################################################################################

image_data <- list.files(file.path(data_dir_root, 'IMAGE'), pattern = 'SSP.*\\.asc.txt$',full.names=TRUE)
image_area <- raster(file.path(data_dir_root, 'IMAGE', 'IMAGE_grid_landarea.asc.txt'))
crs(image_area) <- '+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs'
IM <- stack(image_data) 
crs(IM) <- '+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs'

imageGlobal <- NULL 	
for (layerName in names(IM)) {
	print(paste("Processing IMAGE:", layerName))
	
	regex = '(SSP.*)_([a-z]+)frac_([0-9]+).asc'
	scenario =  gsub(regex, '\\1', layerName)
	lc_type = gsub(regex, '\\2', layerName)
	year =  as.numeric(gsub(regex, '\\3', layerName))
	
	IMGlobal = cellStats(subset(IM, layerName)*image_area, sum)
	IMEU27 <- cellStats(subset(IM, layerName)*image_area*eu27, sum)  
	IMgdt <- data.table(unit = 'km2', 
			year = year, 
			location = 'Global',
			lc_type = lc_type,
			value = IMGlobal,
			model = 'IMAGE',
			scenarioId = scenario)
	IMedt <- data.table(unit = 'km2', 
			year = year, 
			location = 'EU27', 
			lc_type = lc_type, 
			value = IMEU27, 
			model = 'IMAGE', 
			scenarioId = scenario) 
	imageGlobal = rbind(imageGlobal, IMgdt, IMedt)
}

lcTypeConversion <- data.table(lc_type = c('cropland', 'forest', 'grasland', 'othernatural', 'urban'), land_type = c('c', 'f','p', 'u', 'b'))
imageGlobalI <- merge(lcTypeConversion, imageGlobal, by='lc_type')[, c('unit', 'value', 'lc_type') := list('Mha', value*0.0001, NULL)]

write.table(imageGlobalI, file.path(data_dir_root, 'IMAGE/IMAGEGlobal.csv'), row.names=FALSE)


################################################################################################################
### CAPS (previously known as ISAM)
################################################################################################################

## CONVERT AND EXTRACT GLOBAL VALUES 

# list all *.dat files in ISAM folder
isamRootDir = file.path(data_dir_root, "ISAM/LUC4C_Simulations")
isamScenarios = list.files(isamRootDir, pattern='Sim.*', full.names=FALSE)
isamSummary = NULL
isamStack <- stack()
isamEu27 = crop(eu27, extent(-180, 180, -55, 81))


for (scenario in isamScenarios) {
	for (year in as.numeric(list.files(file.path(isamRootDir, scenario), pattern='[0-9]+', full.names=FALSE))) {
		
		print(paste("Processing CAPS:", scenario, year))
		DT = NULL
		
		for (regoinalFile in list.files(file.path(isamRootDir, scenario, year), pattern='\\.dat$', full.names=TRUE)) {
			DT <- rbind(DT, data.table(read.csv(regoinalFile, sep='', header=F))[,1:6, with=FALSE])  # some have an extra 7th column
		}
		setnames(DT,c('ID','latitude','longitude','area','cropland','pasture'))
		
		isamSummary = rbind(isamSummary, DT[, 
						list(unit = "Mha", year = year, location = "Global", model = "CAPS", scenario = scenario, c = sum(cropland)/10^10, p = sum(pasture)/10^10)])
				
		areaRaster <- rasterFromXYZ(DT[, list(longitude,latitude,area)])
		names(areaRaster) <- paste('area', year, scenario)
		cropRaster <- rasterFromXYZ(DT[, list(longitude, latitude, cropland)]) /areaRaster
		names(cropRaster) <- paste('c', year, scenario)
		pastureRaster <- rasterFromXYZ(DT[, list(longitude,latitude,pasture)]) /areaRaster
		names(pastureRaster) <- paste('p', year, scenario)
		
		isamStack = stack(isamStack, cropRaster, pastureRaster)		
		euStack = stack(cropRaster, pastureRaster, areaRaster) * isamEu27
		names(euStack) = c('c', 'p', 'a')
		
		isamSummary = rbind(isamSummary, data.table(as.data.frame(euStack, na.rm=TRUE))[, 
						list(unit = "Mha", year = year, location = "EU27", model = "CAPS", scenario = scenario, c=sum(c*a)/10^10, p=sum(p*a)/10^10)])
	}
}

# write summary values to csv
write.table(melt(isamSummary, variable.name = "land_type", value.name = "value", measure.vars=c('c', 'p')), file.path(data_dir_root, 'ISAM/ISAMsummary.csv'),row.names=FALSE)

# write grids to asc
writeRaster(isamStack, filename=file.path(data_dir_root, 'ISAM/ascii/isam'), format='ascii', bylayer=TRUE, suffix=names(isamStack))

################################################################################################################
### LandSHIFT
################################################################################################################
# Using their non-gridded data submitted

################################################################################################################
### MAgPIE
################################################################################################################

## CONVERT AND EXTRACT GLOBAL VALUES 

# see 'ConvertMAgPIEData.R'

################################################################################################################
### CRAFTY
################################################################################################################

crafty_data <- list.files(file.path(data_dir_root, 'CRAFTY'), pattern='.*LandUseData_[A-Z][0-9]\\.csv$', full.names=TRUE)
agent_map <- data.table(LandUseIndex=seq(-1,5), land_type=c('u','c','c','p','u','f','u'), 
		Landmanager=c("Unmanaged","C_Cereal","NC_Cereal","C_Livestock","NC_Livestock","Forester","Conservationist"),
		factor=c(1,1,1,1,0.27,1,1)) # NC_Livestock is 27% of pasture

for (fileName in crafty_data){
	DTCrafty <- fread(fileName)
	scenario = gsub('.*LandUseData_([A-Z][0-9])\\.csv', '\\1', fileName)
	
	# some files have Landmanager but no LandUseIndex, need to map to common format
	if ("Landmanager" %in% names(DTCrafty) & !"LandUseIndex" %in% names(DTCrafty)) {  
		DTCrafty = merge(DTCrafty, agent_map, by="Landmanager")
	}
	
	## correct for coordinates: x=(x[original]-(-2698874))/1000 #y=(y[original]-(-1855465))/1000
	x_cor <- 2698874
	y_cor <- 1855465
	DTCrafty[, c('X','Y') := list(X*1000-x_cor, Y*1000-y_cor)]

	## convert to raster: projection = WGS72 Albers (taken from DynaClue)
	for (year in seq(2010, 2030, 10)) {
		craftyYear <- rasterFromXYZ(DTCrafty[Tick==year,.(X,Y,LandUseIndex)],crs='+proj=aea +lat_1=32.5 +lat_2=54.5 +lat_0=51.4 +lon_0=22.65 +x_0=0 +y_0=0 +ellps=WGS72 +towgs84=0,0,4.5,0,0,0.554,0.2263 +units=m +no_defs')
		writeRaster(craftyYear, filename=file.path(data_dir_root, 'CRAFTY/ascii', paste(scenario, year, sep = "_")), format='ascii')
	}
}

## Load data to stack
crafty <- stack(list.files(file.path(data_dir_root, 'CRAFTY/ascii'), pattern='\\.asc$', full.names=TRUE))
crs(crafty) <- '+proj=aea +lat_1=32.5 +lat_2=54.5 +lat_0=51.4 +lon_0=22.65 +x_0=0 +y_0=0 +ellps=WGS72 +towgs84=0,0,4.5,0,0,0.554,0.2263 +units=m +no_defs'

## Get European values 
craftyDT = data.table(as.data.frame(crafty, na.rm=TRUE))
CraftyEuropean <- NULL

for (layerName in names(crafty)){
	print (paste("Processing Crafty:", layerName))
	year <- as.numeric(gsub('([A-Z][0-9])_([0-9]+)', '\\2', layerName))
	scenario <- gsub('([A-Z][0-9])_([0-9]+)', '\\1', layerName)
	craftLayer = merge(craftyDT[ , list(LandUseIndex=get(layerName), d=1)], agent_map, by="LandUseIndex")
	DTCraftyI <- craftLayer[, list(unit = 'km2', 
					year = year,
					location = 'EU27',
					model = 'CRAFTY',
					scenario = scenario,
					value=length(d*factor)), by=land_type]
	
	CraftyEuropean <- rbind(CraftyEuropean, DTCraftyI)
}

write.table(CraftyEuropean,file.path(data_dir_root, 'CRAFTY/CRAFTYEurope.csv'),row.names=FALSE)

################################################################################################################
### DynaCLUE
################################################################################################################

## DynaCLUE
dynaclue_data <- list.files(file.path(data_dir_root, 'DynaCLUE'),pattern ='hdr.adf$',full.names=TRUE,recursive=TRUE)
dyna <- stack(dynaclue_data)
## coord. ref: '+proj=aea +lat_1=32.5 +lat_2=54.5 +lat_0=51.4 +lon_0=22.65 +x_0=0 +y_0=0 +ellps=WGS72 +towgs84=0,0,4.5,0,0,0.554,0.2263 +units=m +no_defs'
names(dyna) <- gsub('.*/[A-z]+_[A-Z][0-9]/(.*)/hdr.adf', '\\1', dynaclue_data)

## write grids to asc
writeRaster(dyna, filename=file.path(data_dir_root, 'DynaCLUE/ascii/dyna'), format='ascii', bylayer=TRUE, suffix=names(dyna))

#DynaRclMatrix <- matrix(c(c(seq(0,17,1),255), c(0,1,2,3,9,9,4,5,6,-99,7,9,9,9,9,9,8,-99,NA)), ncol=2)
DynaRclMatrixI <- matrix(c(c(seq(0,17,1),255), c(0,1,2,4,4,4,1,4,1,1,3,4,4,4,4,4,4,1,NA)), ncol=2)
dynaRecl <- reclassify(dyna, rcl=DynaRclMatrixI)
names(dynaRecl) <- names(dyna)


# the 2000 data contains countries other than EU27, so need to mask these out.  The later results are used to create this mask
euDynaMask = !is.na(subset(dynaRecl, "ref_a1_2020"))
for (layerName in data.table(names(dynaRecl))[V1 %like% '2000']) { # could do the whole stack, but quicker just to do the ones we need
	maskedLayer = euDynaMask * subset(dynaRecl, layerName)
	names(maskedLayer) = layerName
	dynaRecl = dropLayer(dynaRecl, layerName)
	dynaRecl = stack(dynaRecl, maskedLayer)
}

DynaEuropean <- NULL
for (layerName in names(dynaRecl)){
	print (paste("Processing DynaCLUE:", layerName))
	
	scenario =  gsub('ref_(.*)_[0-9]+', '\\1', layerName)
	year =  as.numeric(gsub('ref_[a-z][0-9]_(*)', '\\1', layerName))
	layerized <- layerize(subset(dynaRecl, layerName)) 
	
	built <- subset(layerized, 'X0')
	cropland <- subset(layerized, 'X1')
	pasture <- subset(layerized, 'X2')
	forest <- subset(layerized, 'X3')
	unmanaged <- subset(layerized, 'X4')
	
	DTDyna <- data.table(unit = 'km2',
			year = year,
			location = 'EU27',
			model = 'DynaCLUE',
			scenario = scenario,
			b = cellStats(built, sum),
			c = cellStats(cropland, sum), 
			p = cellStats(pasture, sum),
			f = cellStats(forest, sum),
			u = cellStats(unmanaged, sum))
	DynaEuropean <- rbind(DynaEuropean, DTDyna)
}
DynaEuropeanData <- melt(DynaEuropean, variable.name = "land_type", value.name = "value", measure.vars=c('b', 'c', 'p', 'f', 'u'))

write.table(DynaEuropeanData,file.path(data_dir_root, 'DynaCLUE/DynaEurope.csv'),row.names=FALSE)

################################################################################################################
### Ecochange/ALARM
################################################################################################################
ecochange_data <- list.files(file.path(data_dir_root, 'ECOCHANGE_ALARM'),pattern ='hdr.adf$',full.names=TRUE,recursive=TRUE)
ecochange <- stack(ecochange_data)
names(ecochange) <- gsub('.*/ECOCHANGE_ALARM/[A-Z]+/(.*)/hdr.adf', '\\1', ecochange_data)
crs(ecochange) <- '+proj=laea +lat_0=52 +lon_0=10 +x_0=4321000 +y_0=3210000 +ellps=GRS80 +units=m +no_defs' 

EcochangeEu <- NULL

for (layerName in names(ecochange)){
	
	print (paste("Processing Ecochange: ", layerName))
	
	if (layerName == "baseline_eu") {
		scenario = "baseline"
		year = 2010
	}
	else {
		scenario =  gsub('([a-z]+)_[0-9]+', '\\1', layerName)
		year =  as.numeric(gsub('[a-z]+_([0-9]+)', '\\1', layerName))
	}
	
	DTEcochange <- data.table(unit = 'km2',
			year = year,
			location = 'EU25+CH+NO',
			model = 'Ecochange',
			scenario = scenario,
			b = factorValues(subset(ecochange, layerName), 1)$COUNT*(0.25*0.25),
			c = (factorValues(subset(ecochange, layerName), 2)$COUNT + factorValues(subset(ecochange, layerName), 3))$COUNT*(0.25*0.25),
			p = factorValues(subset(ecochange, layerName), 4)$COUNT*(0.25*0.25), # Attention! Includes also natural grassland, moors and heathland and shrub transitional!
			f = factorValues(subset(ecochange, layerName), 5)$COUNT*(0.25*0.25),
			u = factorValues(subset(ecochange, layerName), 6)$COUNT*(0.25*0.25)
	)
	EcochangeEu <- rbind(EcochangeEu, DTEcochange)
}

# add baseline to each scenario
EcochangeEu = rbind(EcochangeEu[scenario != "baseline"], 
		EcochangeEu[scenario == "baseline", list(unit,year,location,model,scenario=unique(EcochangeEu[scenario != "baseline"]$scenario), b, c, p, f, u)])

EcochangeEuData <- melt(EcochangeEu, variable.name = "land_type", value.name = "value", measure.vars=c('b', 'c', 'p','f', 'u'))
write.table(EcochangeEuData,file.path(data_dir_root, 'ECOCHANGE_ALARM/EcochangeEurope.csv'),row.names=FALSE)

################################################################################################################
### CLIMSAVE
################################################################################################################

climsaveShp = readOGR(dsn=file.path(data_dir_root, "Climsave/Climsave_European_Cells_ID"), layer = "Climsave_European_Cells_ID")
climsaveGrid = data.table(as.data.frame(climsaveShp), key='Cell_ID')[, list(Cell_ID, GRID_AREA_, Country)]
climsaveFiles = list.files(file.path(data_dir_root, 'Climsave'), pattern='^.+Forest.csv$')

climsaveEuData = NULL
for (fileName in climsaveFiles) {
	print (paste("Processing CLIMSAVE: ", fileName))
	
	regex = '([a-zA-Z0-9]+)_([a-zA-Z0-9]+)_*[SRES]*([a-zA-Z0-9]*)_.*Forest.csv'
	climateScenario =  gsub(regex, '\\1', fileName)
	climateModel =  gsub(regex, '\\2', fileName)
	economicScenario =  gsub(regex, '\\3', fileName)
	
	if (economicScenario == "") year = 2010 else year = 2050
	
	dt = fread(file.path(data_dir_root, 'Climsave', fileName), autostart=150)
	dt = dt[, c("cellID", "Intensively farmed (%)", "Arable crops (%)", "Managed forest (%)", "Unmanaged forest (%)", "Unmanaged land (%)", "Urban (%)", "Permanent grass area (ha)", "Extensive grass area (ha)"), with=FALSE]
	setnames(dt, c('cellID', 'int_p', 'c_p', 'fm_p', 'fu_p', 'u_p', 'b_p', 'perm_grass', 'ext_grass'))
	setkey(dt, cellID)
	
	dt = dt[climsaveGrid, list(cellID, Country, 
					c = c_p * GRID_AREA_/100,
					fm = fm_p * GRID_AREA_/100,
					fu = fu_p * GRID_AREA_/100,
					u = u_p * GRID_AREA_/100,
					b = b_p * GRID_AREA_/100, 
					pi = (int_p - c_p) * GRID_AREA_/100,
					pe = ext_grass,
					GRID_AREA_)]
	
	dt = dt[!Country %in% c("CH", "NO")]  # remove Swizterland and Norway (but still missing Cyprus)
	dt2 = dt[, c('cellID', 'GRID_AREA_') := list(NULL, NULL)][, lapply(.SD, sum, na.rm=TRUE), by=Country]
	dt3 = melt(dt2, variable.name = "land_type", value.name = "value", measure.vars=c("c","fm","fu","u","b","pi","pe"))
	
	climsaveEuData = rbind(climsaveEuData, dt3[, list(model = "Climsave", year = year, location = Country, economicScenario, climateScenario, climateModel, land_type, value = value /10^6, unit = "Mha")])
}

climSum = climsaveEuData[, list(value = sum(value), location = "EU27"), by=list(model, year, economicScenario, climateScenario, climateModel, land_type, unit)]

# add 2010 baseline to each scenario
a = unique(climSum[year != 2010, list(economicScenario, climateScenario, climateModel)])
b = climSum[year == 2010][, c('economicScenario', 'climateScenario', 'climateModel') := list(NULL, NULL, NULL)]
c = rbind(climSum[year != 2010], a[,as.list(b), by=list(economicScenario, climateScenario, climateModel)])

write.table(c, file.path(data_dir_root, 'Climsave/ClimsaveEurope.csv'),row.names=FALSE)


################################################################################################################
### LUMP/LUISA 
################################################################################################################

## original maps masked with country boundaries data EU27 

luisa_data <- list.files(file.path(data_dir_root, 'LUMP_LUISA/ArcGIS_EU27'), pattern='\\.txt$', full.names=TRUE)

luisaRecl <- data.table(original = seq(0,22,1), land_type = c('none','b','b','c','c','p','f','u','c','c','c','c','c','p','b','b','c','u','b','u','u','u','u'))

LUISAEU27 <- NULL
for (entry in luisa_data){
	year <- as.numeric(gsub('.*/LUMP_LUISA/ArcGIS_EU27/(.*).txt', '\\1', entry))
	dt <- fread(entry)
	dtI <- dt[, 'OBJECTID' := NULL]
	setnames(dtI, 'Value', 'original')
	dtII <- merge(dtI, luisaRecl, by='original')[, 'original' := NULL]
	#setkey(dtII, land_type)
	dtIII <- dtII[, .(land_type_sum = sum(Count)), by=land_type]
	dtIII[, c('unit', 'year', 'location', 'land_type', 'value', 'model', 'scenario', 'land_type_sum') := list('km2', year, 'EU27', land_type, land_type_sum*(0.01), 'LUISA', 'reference', NULL)]
	dtIV <- dtIII[land_type != 'none' ]
	LUISAEU27 <- rbind(LUISAEU27, dtIV)
}

write.table(LUISAEU27, file.path(data_dir_root, 'LUMP_LUISA/LUISASummary.csv'), row.names=FALSE)

