require(data.table)
require(reshape2)

data_dir_root = "/Users/peteralexander/Documents/LURG/Model Comp/Phase1/LUC4C MIP"

convertAreaUnitsToMha = function(dtToConv) {
	land_unit_conv_to_Mha=data.table(unit = c('km2', '1000 ha', 'Mha', 'Gha'), factor = c(0.0001, 0.001, 1, 1000), key = 'unit')
	merge(land_unit_conv_to_Mha, dtToConv, by="unit")[, c('value', 'unit', 'factor') := list (value * factor, 'Mha', NULL)]
}

convertScenarios = function(dtToConv) {
	sspSresData = fread(file.path(data_dir_root, "sspSresData.csv"))
	rcpData = fread(file.path(data_dir_root, "rcpData.csv"))
	
	# add economic scenario data
	scenarioDataConv = merge(dtToConv, sspSresData[, list(economicScenario, pop_2100, gdp_rate_to_2100, inequality_2100, tech_change, trade)], by='economicScenario', all.x=TRUE)
	
	# add climate scenario data
	scenarioDataConv = merge(scenarioDataConv, sspSresData[, list(climateScenario=economicScenario, rcp, base_co2_2100 = co2_2100)], by='climateScenario', all.x=TRUE)
	
	scenarioDataConv[climateScenario %like% 'RCP' | climateScenario == "present climate", rcp := climateScenario]
	scenarioDataConv = merge(scenarioDataConv, rcpData, by='rcp', all.x=TRUE)	
	scenarioDataConv[!(climateScenario %like% 'RCP' | climateScenario == "present climate"), co2_2100 := base_co2_2100]
	scenarioDataConv[, base_co2_2100 := NULL]
	
	scenarioMapping = data.table(
			economicScenario = c('A1', 'A2', 'B1', 'B2'), 
			ssp = c('SSP5', 'SSP3', 'SSP1', 'SSP2'), key='economicScenario')	
	scenarioDataConv = merge(scenarioDataConv, scenarioMapping, by="economicScenario", all.x=TRUE)
	scenarioDataConv[, groupedScenario := economicScenario]
	scenarioDataConv[!is.na(ssp), groupedScenario := ssp]
	scenarioDataConv[model == "FABLE", groupedScenario := "SSP5"] # all FABLE scenarios basd on A1B, which we map to SSP5
	scenarioDataConv[, ssp := NULL]
	
	scenarioDataConv
}

readDataByRegion = function(file, model, scenarioId, dtToAddTo = NULL) {
	# read the data
	newData = fread(file.path(data_dir_root, file))[,1:5, with=FALSE]  # just takes the first 5 columns something there is a trailing comma
	setnames(newData, c("year", "location", "land_type", "unit", "value"))
	newData[, c('year', 'value', 'model', 'scenarioId') := list(as.numeric(year), as.numeric(value), model, scenarioId)]
	
	newData = convertAreaUnitsToMha(newData)
	
	# some data clean-up
	newData[land_type == "Cropland" | land_type == "C", land_type := "c"]
	newData[land_type == "Pasture", land_type := "p"]
	newData[land_type %like% "Urban", land_type := "b"]
	newData[land_type == "Managed forest", land_type := "fm"]
	
	newData[location == "global" | location == "WLD" | location == "World", location := "Global"]
	
	# append the data, if needed
	if (is.null(dtToAddTo))
		newData
	else
		rbind(dtToAddTo, newData)
}

addScenario = function(scenarioDt, model, scenarioId, economicScenario, climateScenario, climateModel) {
	rbind(scenarioDt, data.table(model=model, scenarioId=scenarioId, economicScenario=economicScenario, climateScenario=climateScenario, climateModel=climateModel)) 
}

readNonGriddedData = function() {
	# allData is the table were we are going to hold all non-gridded data, for all scenarios and models
	allData = NULL
	eu27Data = NULL
	scenarioData = NULL
	
	eu27Land = 418.172	# Using LAND area at 2011 FAOStat (2014),  (not country area)
	croatiaLand = 5.596
	romaniaBulgariaLand = 23.016 + 10.856 
	area_ratio_eu25_to_eu27 = eu27Land/(eu27Land - romaniaBulgariaLand); 
	area_ratio_eu28_to_eu27 = eu27Land/(eu27Land + croatiaLand);
	area_ratio_farm_eur_to_eu27 = eu27Land/(eu27Land + croatiaLand + 2.74 + 2.522 + 8.746 + 1.355 + 5.1); # albania, macedonia, serbia, montenegro , bosnia
	area_ratio_ecochange_to_eu27 = eu27Land/(eu27Land - romaniaBulgariaLand + 36.5268 + 3.9516); # norway and switzerland
	
	# AIM
	for (scenario in sprintf("SSP%d", c(1:3))) {
		allData = readDataByRegion(sprintf("AIM/new/ng_%s_2005.csv", scenario), "AIM", scenario, allData)
		scenarioData = addScenario(scenarioData, "AIM", scenario, scenario, "present climate", "AIM-Unknown")
	}
	eu27Data = rbind(eu27Data, allData[model == "AIM" & location == "XE25"][, c('location', 'value') := list("EU27", value*area_ratio_eu25_to_eu27)])
	
	# FABLE
	for (scenario in sprintf("scenario_%d", c(1:6))) {
		allData = readDataByRegion(sprintf("FABLE/%s_2005.csv", scenario), "FABLE", scenario, allData)
		
		if (scenario == "scenario_2")
			fableScenario = "FableB" # RCP 8.5
		else if (scenario == "scenario_4")
			fableScenario = "FableC" # lower gdp growth
		else if (scenario == "scenario_5")
			fableScenario = "FableD" # higher population
		else
			fableScenario = "FableA"
		
		scenarioData = addScenario(scenarioData, "FABLE", scenario, fableScenario, if(scenario == "scenario_2") "RCP 8.5" else "RCP 2.6", "Fable-Unknown")
	}
	
	# FALAFEL
	falafelData = fread(file.path(data_dir_root, "FALAFEL/FALAFEL.csv"))
	falafelData = melt( falafelData, measure.vars = c('c', 'pi', 'pe', 'fm', 'fu', 'b', 'uv', 'un'), variable.name = "land_type", value.name = "value")
	falafelData = falafelData[,list(year, location="Global", land_type, unit="Mha", value, model="FALAFEL", scenarioId = toupper(scenario))]
	
	scenarioData = rbind(scenarioData, unique(falafelData[, list(model="FALAFEL", scenarioId, economicScenario=scenarioId, climateScenario="present climate", climateModel="Falafel-Unknown")]))
	allData = rbind(allData, falafelData)
	
	# FARM
	farmScenarios = fread(file.path(data_dir_root, "FARM/scenarios.txt"))
	farmScenarios[, c('model', 'climateScenario', 'climateModel') := list("FARM", gsub('(RCP [0-9.]+) [a-zA-Z]+', '\\1', climate), gsub('RCP [0-9.]+ ([a-zA-Z]+)', '\\1', climate))]
	setnames(farmScenarios, c('id', 'SSP'), c('scenarioId', 'economicScenario'))
	farmData = NULL
	
	for (scenario in farmScenarios$scenarioId) {
		farmData = readDataByRegion(sprintf("FARM/ng_%s_2010.csv", scenario), "FARM", scenario, farmData)
	}
	
	farmAvgClimateModel = merge(farmData, farmScenarios, by=c("scenarioId", "model"))[, 
			list(scenarioId = paste(economicScenario, climateScenario), climateModel = "Multi-mean", value=mean(value)), 
			by=list(model, economicScenario, climateScenario, land_type, year, location, unit)]
	
	allData = rbind(allData, farmAvgClimateModel[, list(unit, year, location, land_type, value, model, scenarioId)])
	scenarioData = rbind(scenarioData, unique(farmAvgClimateModel[, list(model, scenarioId, economicScenario, climateScenario, climateModel)]))
	eu27Data = rbind(eu27Data, allData[model == "FARM" & location == "EUR"][, c('location', 'value') := list("EU27", value=value*area_ratio_farm_eur_to_eu27)])
	
	# GCAM - got LU and LC types in dataset (care needed)
	gcamData = unique(fread(file.path(data_dir_root, "GCAM/luc4c_GCAM_output.csv")))
	setnames(gcamData, "scenario", "scenarioId")
	scenarioData = rbind(scenarioData, unique(gcamData[, list(model, scenarioId, economicScenario=scenarioId, climateScenario="present climate", climateModel="None")]))
	allData = rbind(allData, gcamData, use.names = TRUE)
	eu27Data = rbind(eu27Data, allData[model == "GCAM" & location %in% c("EU-12", "EU-15"), list(location = "EU27", value=sum(value)), by=list(unit, year, land_type, model, scenarioId)])
	
	#IMAGE - lets take the values from the gridded data
	imageData = fread(file.path(data_dir_root, "IMAGE/IMAGEGlobal.csv"))
	allData = rbind(allData, imageData)
	imageScenarios = unique(imageData[,list(scenarioId)])[, list(model='IMAGE', scenarioId, economicScenario=gsub('(SSP[0-9])_.*', '\\1', scenarioId),
					climateScenario="RCP 2.6", climateModel='IPSL_CM4A')]
	scenarioData = rbind(scenarioData, imageScenarios)
	
	# MAGNET
	magnetMapping = data.table(ssp = c("SSP1", "SSP2", "SSP3"), rcp = c("RCP 4.5", "RCP 6.0", "RCP 8.5"))
	for (scenario in sprintf("SSP%d", c(1:3))) {
		allData = readDataByRegion(sprintf("MAGNET/ng_%s_2007.csv", scenario), "MAGNET", scenario, allData)
		scenarioData = addScenario(scenarioData, "MAGNET", scenario, scenario, magnetMapping[ssp==scenario]$rcp, "Magnet-Unknown")
	}
	
	magnetEuData = allData[model == "MAGNET" & location %in% c("EU12", "EU16"), list(location="EU27",  value=sum(value)*area_ratio_eu28_to_eu27), by=list(unit, year, land_type, model, scenarioId)]
	eu27Data = rbind(eu27Data, magnetEuData)
	
	# PLUM
	for (scenario in c('A1', 'A2', 'B1', 'B2')) {
		allData = readDataByRegion(sprintf("PLUM/ng_%s_1990.csv", scenario), "PLUM", scenario, allData)
		scenarioData = addScenario(scenarioData, "PLUM", scenario, scenario, scenario, "PLUM-Unknown")
	}
	plumEu = allData[model == "PLUM" & location %in% fread(file.path(data_dir_root, 'eu27.txt'))$alphaCode,  list(location="EU27", value=sum(value)), by=list(unit, year, land_type, model, scenarioId)]
	eu27Data = rbind(eu27Data, plumEu)
	
	#CLUMondo
	cluData = fread(file.path(data_dir_root,'CLUMondo/CLUglobal.csv'))[scenarioId != "OECD"] # OECD scenario is older version of CLUMondo
	scenarioData = addScenario(scenarioData, "CLUMondo", "CARBON", "SSP2", "present climate", "None")
	scenarioData = addScenario(scenarioData, "CLUMondo", "FAO4D", "SSP2", "present climate", "None")
	scenarioData = addScenario(scenarioData, "CLUMondo", "OECD", "SSP2", "present climate", "None")
	scenarioData = addScenario(scenarioData, "CLUMondo", "PPA", "SSP2", "present climate", "None")
	allData = rbind(allData, cluData)
	
	# CAPS (previously known as ISAM)
	capsData = fread(file.path(data_dir_root, 'ISAM/ISAMsummary.csv'))
	setnames(capsData, "scenario", "scenarioId")
	capsScenarioData = fread(file.path(data_dir_root, 'ISAM/scenarios.txt'))[, model := "CAPS"]
	capsScenarioData[economicScenario == "RCP 4.5", economicScenario := "B1"]
	capsScenarioData[economicScenario == "RCP 8.5", economicScenario := "A2"]
	scenarioData = rbind(scenarioData, capsScenarioData)
	allData = rbind(allData, capsData)
	
	# LandSHIFT
	landShiftData = NULL
	for (scenario in c('FUEL_BAU', 'FUEL_REGULATIONS', 'HEAT_BAU', 'HEAT_REGULATIONS')) {
		
		for (fileName in list.files(file.path(data_dir_root, "LandShift/global", scenario), pattern='Scen.*', full.names=TRUE)) {	
			year =  gsub('.*/Scen_.*_([0-9]+).txt', '\\1', fileName)
			dt = fread(fileName)
			setnames(dt, gsub('-', '', (names(dt))))  # remove annoying hyphen from set-aside
			
			dt = dt[, list(year = Year, location = GeoRegion, 
							b = urban, c = crops, pi = pasture, pe = grassland + savannah + shrubland, f = forest, u =  + barren + wetland)]
			dt = melt( dt, variable.name = "land_type", value.name = "value", measure.vars=c("b", "c", "pi", "pe", "f","u"))[, scenario := scenario]
			landShiftData = rbind(landShiftData, dt[,list(year, location, land_type, unit = "Mha", value=value/10000, model = "LandSHIFT", scenarioId=scenario)])
		}
		
		scenarioData = addScenario(scenarioData, "LandSHIFT", scenario, "SSP2", "Baseline", "None")
	}
	
	allData = rbind(allData, landShiftData)	
	eu27Data = rbind(eu27Data, landShiftData[location %in% c("Germany", "France", "United Kingdom", "Italy", "Poland", "Benelux", "Scandinavia and Baltics", "Rest of EU15", "Rest of new European Member States (2004)", "EU2 Bulgaria and Romania"), 
					list(location = "EU27", value = sum(value)), by=list(year, land_type, unit, model, scenarioId)])
	
	# MAgPIE - this is reading the converted data
	magpieData = fread(file.path(data_dir_root, "MAgPIE/MAgPIEsummary.csv"))
	setnames(magpieData, "scenario", "scenarioId")
	scenarioData = rbind(scenarioData, unique(magpieData[, list(model, scenarioId, economicScenario="SSP2", climateScenario="present climate", climateModel="None")]))
	allData = rbind(allData, magpieData)
	
	# GLOBIOM
	globiomData = fread(file.path(data_dir_root, 'GLOBIOM/summary.csv'))
	globiomData = unique(globiomData) # some duplicates !
	globiomData[, year := as.numeric(year)]
	scenarioData = rbind(scenarioData, unique(globiomData[, list(model, scenarioId, economicScenario=scenarioId, climateScenario="present climate", climateModel="None")]))
	allData = rbind(allData, globiomData)
	
	
	# Calc global totals for global model
	modelsWithoutGlobals = unique(allData[!model %in% unique(allData[location == "Global", model]), model])
	dt = allData[model %in% modelsWithoutGlobals, list(value = sum(value), location = "Global"), by = list(model, scenarioId, year, land_type, unit)]
	allData = rbind(allData, dt, use.names=TRUE)
	
	
	######### EUROPEAN MODELS
	# CRAFTY	
	# old way using summed gridded data
	#	craftyEuData = fread(file.path(data_dir_root,'CRAFTY/CRAFTYEurope.csv'))
	#	setnames(craftyEuData, "scenario", "scenarioId")
	
	# new way, using summed data file
	agent_map <- data.table(land_type=c('u','c','c','p','p','f','u'), 
			Landmanager=c("Unmanaged","C_Cereal","NC_Cereal","C_Livestock","NC_Livestock","Forester","Conservationist"),
			factor=c(1,1,1,1,0.27,1,1))
	DTCrafty = NULL
	for (fileName in list.files(file.path(data_dir_root, 'CRAFTY'), pattern='.*LandUseData_sums_[A-Z][0-9]\\.csv$', full.names=TRUE)){
		scenario = gsub('.*LandUseData_sums_([A-Z][0-9])\\.csv', '\\1', fileName)
		DTCrafty <- rbind(DTCrafty, fread(fileName)[, scenarioId := scenario])
	}
	DTCrafty = merge(DTCrafty, agent_map, by="Landmanager")
	craftyEuData = DTCrafty[, list(unit="km2", location="EU27", model="CRAFTY", value=sum(AFT*factor)), by=list(land_type, year=Tick, scenarioId)]
	
	craftyEuData = convertAreaUnitsToMha(craftyEuData)
	scenarioData = rbind(scenarioData, unique(craftyEuData[, list(model, scenarioId, economicScenario=scenarioId, climateScenario=scenarioId, climateModel="CRAFTY-Unknown")]))
	eu27Data = rbind(eu27Data, craftyEuData)
	
	# DynaCLUE
	dynaEuData = fread(file.path(data_dir_root,'DynaCLUE/DynaEurope.csv'))
	dynaEuData = convertAreaUnitsToMha(dynaEuData)
	setnames(dynaEuData, "scenario", "scenarioId")
	dynaEuData[, scenarioId := toupper(scenarioId)]
	scenarioData = rbind(scenarioData, unique(dynaEuData[, list(model, scenarioId, economicScenario=scenarioId, climateScenario=scenarioId, climateModel="Dyna-Unknown")]))
	eu27Data = rbind(eu27Data, dynaEuData)
	
	#Ecochange/ALARM
	ecochangeEuData = fread(file.path(data_dir_root,'ECOCHANGE_ALARM/EcochangeEurope.csv'))
	ecochangeEuData = convertAreaUnitsToMha(ecochangeEuData)
	setnames(ecochangeEuData, "scenario", "scenarioId")
	ecochangeEuData[, c('model', 'value', 'location', 'scenarioId') := list('EcoChange', value * area_ratio_ecochange_to_eu27, "EU27", toupper(scenarioId))]
	ecochangeScenarios = fread(file.path(data_dir_root, "ECOCHANGE_ALARM/scenarios.txt"))
	ecochangeScenarios[, model := 'EcoChange']
	scenarioData = rbind(scenarioData, ecochangeScenarios)
	eu27Data = rbind(eu27Data, ecochangeEuData)
	
	# Climsave
	climsaveEuData = fread(file.path(data_dir_root,'Climsave/ClimsaveEurope.csv'))
	# calc mean of climate models
	# climsaveEuData[, scenarioId := paste(economicScenario, climateScenario, climateModel)]
	climsaveEuData = climsaveEuData[, list(model = "CLIMSAVE-IAP", scenarioId = paste(economicScenario, climateScenario), climateModel = "Multi-mean", value=mean(value)), 
			by=list(economicScenario, climateScenario, land_type, year, location, unit)]
	
	scenarioData = rbind(scenarioData, unique(climsaveEuData[, list(scenarioId , model, economicScenario , climateScenario, climateModel)]))
	
	climsaveEuData[, c('economicScenario', 'climateScenario', 'climateModel') := list(NULL, NULL, NULL)]
	eu27Data = rbind(eu27Data, climsaveEuData)
	
	# LUMP/LUISA
	luisaEuData = fread(file.path(data_dir_root, 'LUMP_LUISA/LUISASummary.csv'))
	luisaEuData = convertAreaUnitsToMha(luisaEuData)
	setnames(luisaEuData, 'scenario', 'scenarioId')
	scenarioData = addScenario(scenarioData, 'LUISA', 'reference', 'SSP2','RCP 6.0','LUISA-Unknown')
	eu27Data = rbind(eu27Data, luisaEuData)
	
	
	# At the moment only interested in Global and EU27
	allData = rbind(allData[location %in% c("Global", "EU27")], eu27Data)
	
	# Create land cover rows from sub types if they don't already exist	
	land_types_map = data.table(cover = c('c', 'c', 'c', 'f', 'f', 'p', 'p', 'u', 'u'), use = c('an', 'ai', 'pc', 'fm', 'fu', 'pi', 'pe', 'uv', 'un'))
	
	for (land_cover in unique(land_types_map$cover)) {
		land_uses = land_types_map[cover == land_cover, use]
		
		allData = rbind(allData, allData[!model %in% unique(allData[land_type==land_cover, model]) & land_type %in% land_uses, 
						list(land_type = land_cover, value=sum(value)), by=list(unit, year, location, model, scenarioId)])
	}
	
	interpolatedData = interpolateMissingYears(allData)
	allData = rbind(allData[,dummy:=FALSE], interpolatedData[,dummy:=TRUE])
	
	# rebase
	baseMean = allData[year == 2010, list(mean2010 = mean(value), median2010 = median(value)), by=list(location, land_type)]
	baseHist = fread(file.path(data_dir_root, 'histSummary.csv'))[year == 2010, list(location, land_type, v2010=value)]
	base2010 = rbind(baseMean[!land_type %in% c('c', 'p', 'f'), list(land_type, location, v2010=mean2010)], baseHist)
	base2010 = merge(baseMean, base2010, by=c('land_type', 'location'))	
	
	scen2010 = merge(allData[year == 2010], base2010, by=c('land_type', 'location'))
	scen2010 = scen2010[, list(location, land_type, model, scenarioId, mean2010, median2010, scenAdj = v2010/value, scenDelta = value - v2010)]
	allData = merge(allData, scen2010, by=c('location', 'land_type', 'model', 'scenarioId'))[, c('value_rebased_factor', 'value_rebased') := list(value * scenAdj, value-scenDelta)]

	# a check: allNonGriddedData[land_type %in% c('c', 'f', 'p') & abs((value_rebased_factor-value_rebased) /value_rebased) > 0.14, list(model, land_type, scenarioId, year, value, scenAdj, scenDelta, value_rebased, value_rebased_factor, (value_rebased_factor-value_rebased) /value_rebased)]

	# Convert SRES to SSP
	scenarioDataConverted = convertScenarios(scenarioData)
	#scenarioDataConverted[, list(model, scenarioId, economicScenario, climateScenario, pop_2100, gdp_rate_to_2100, inequality_2100, co2_2100, rcp, groupedScenario)]
	
	# Add model and scenario details
	modelData = fread(file.path(data_dir_root, "modelData.txt"))
	
	if (length(allData[!model %in% modelData$model]$model) > 0) print ("*********** MISSING MODEL DATA")
	allData = merge(allData, scenarioDataConverted, by=c('model', 'scenarioId'))
	allData = merge(allData, modelData, by='model')
	allData
}

# There must be a better way (without the 'for' loop) to do this, but unfortunately I can't think of it right now....
interpolateMissingYears = function(dtToAvg) {	
	modelYears = dtToAvg[, list(year, model, scenarioId, aval=TRUE)]
	requiredModelYears = data.table(merge.data.frame(unique(modelYears[, list(model, scenarioId)]), data.table(year = seq(2010, 2100, 10)), all=TRUE))
	missingModelYears = merge(requiredModelYears, modelYears, by=c("year", "model", "scenarioId"), all.x=TRUE)[is.na(aval)][, aval := NULL]
	ptsToInt = merge(missingModelYears, dtToAvg[, list(maxYear=max(year)), by=list(model, scenarioId)], by=c('model', 'scenarioId'))[year < maxYear]
	
	interpolatedDt = NULL
	
	if (length(ptsToInt$year) > 0)
		for (i in 1:length(ptsToInt$year)) {
			interpolatedDt = rbind(interpolatedDt, interpolatePoint(dtToAvg, ptsToInt[i]$model, ptsToInt[i]$scenarioId, ptsToInt[i]$year))
		}
	interpolatedDt
}

interpolatePoint = function(dtToInt, mod, scenId, yr) {	
	dt = dtToInt[model == mod & scenarioId == scenId]
	lowerYear = dt[year < yr, max(year)]
	upperYear = dt[year > yr, min(year)]
	
	idt = merge(dt[year == lowerYear], dt[year == upperYear], by = c('model', 'scenarioId', 'land_type', 'location'), suffixes = c("", ".y"))
	idt[, list(model, scenarioId, land_type, location, year=yr, unit,
					value = value + (yr-year)/(year.y-year)*(value.y-value))]
}

allNonGriddedData = readNonGriddedData()
setkey(allNonGriddedData, model, scenarioId, year)

# removing two scenarios as believe not form part of the sample results sample
allNonGriddedData = allNonGriddedData[!(model == "MAgPIE" & scenarioId %in% c('AFF', 'AFF+BECCS')) & !(model == "IMAGE" & scenarioId %in% c('SSP1_450_ADAFF'))]
#allNonGriddedData = allNonGriddedData[!(model == "MAgPIE" & scenarioId %in% c('AFF', 'AFF+BECCS', 'BECCS')) & !(model == "IMAGE" & scenarioId %in% c('SSP1_450_ADAFF', 'SSP2_450_BECCS'))]


#write.table(allNonGriddedData, "/Users/peteralexander/Documents/LURG/Model Comp/Phase1/LUC4C MIP/allNonGriddedData.csv", sep=',', row.names=FALSE, quote=FALSE)

#dt = scenarioDataConverted[, list(model, scenarioId, economicScenario, climateScenario, pop_2100, gdp_rate_to_2100, inequality_2100, co2_2100, tech_change, trade)]
#setkey(dt, model, scenarioId)
#write.table(dt, "/Users/peteralexander/Documents/LURG/Model Comp/Phase1/scenarioSummary_15July15.csv", sep=',', row.names=FALSE, quote=FALSE)