require(data.table)
require(reshape2)
require(zoo)

readLivestockNumbers = function() {
	animal_prod_data = fread(file.path(fao_data_dir,"Production_LivestockPrimary_E_All_Data_Norm.csv"))
	if ("Area" %in% names(animal_prod_data)) setnames(animal_prod_data, "Area", "Country")
	
	livestockNumMapping = data.table(plumDemandItem = c(rep("Monogastrics",3), rep("Ruminants",7)),
			Item = c("Eggs Primary", "Meat Poultry", "Meat pig", "Milk whole fresh cow", "Milk whole fresh sheep", "Milk whole fresh goat", 
					"Meat cattle", "Meat buffalo", "Meat sheep", "Meat goat"),
			adjustToMillionHead = c(1000, 1000, rep(1000000, 8)))
	
	dt = merge(animal_prod_data[Element %in% c("Producing Animals/Slaughtered", "Milk Animals", "Laying")], livestockNumMapping, by="Item")
	dt[Country == "Eswatini", Country:= "Swaziland"] # some FAO dataset contain old name of Swaziland, and some new name of Eswatini
	dt[, list(plumDemandItem, LivestockItem=Item, Country, Year, animalM=Value/adjustToMillionHead)]
}

# Read SSP data
getSsp  = function(country_data) {
	ssp_data = fread(file.path(data_dir_root,"SSP","SspDb_country_data_2013-06-12.csv"), header=TRUE)
	
	ssp_pop = reshape(ssp_data[VARIABLE=="Population"], varying = as.character(seq(2010,2100,5)), 
			v.names = "population", timevar = "Year", 
			times = seq(2010,2100,5), direction = "long")
	
	ssp_gdp = reshape(ssp_data[VARIABLE=="GDP|PPP"], varying = as.character(seq(2010,2100,5)), 
			v.names = "gdp_ppp", timevar = "Year", 
			times = seq(2010,2100,5), direction = "long")
	
	ssp_dt=merge(ssp_pop, ssp_gdp, by=c("MODEL", "SCENARIO", "REGION", "Year"))[,list(Model=MODEL, Scenario=SCENARIO, CountryCode=REGION, Year, gdp_ppp=gdp_ppp*1000, population=population)]
	
	baseline_gdp = country_data[Year == 2010]
	baseline_gdp_adj = merge(baseline_gdp, ssp_dt[Year == 2010], by=c("CountryCode"))[,list(CountryCode, Country, Model, Scenario, IncomeGroup, gdp_adj=gdp/gdp_ppp, pop_adj=population.x/population.y)]
	ssp_dt = merge(ssp_dt, baseline_gdp_adj, by=c("CountryCode", "Model", "Scenario"))
	ssp_dt[, c('gdp', 'population') := list(gdp_ppp * gdp_adj, population * pop_adj)]
	
	ssp_dt[Country == "Sudan", Country:="Sudan (former)"]
	ssp_dt[Model != "PIK GDP-32", list(Model, Scenario, Country, Year, gdp, population)] # PIK GDP-32 seems to be US only
}

disaggregateCountries = function(Dt,wbCountryDt, colsToChange){
	
	splitting_states = data.table (oldC="USSR", newC = c("Armenia", "Azerbaijan", "Belarus", "Estonia", "Georgia", "Kazakhstan", "Kyrgyzstan", "Latvia", 
					"Lithuania", "Republic of Moldova", "Russian Federation", "Tajikistan", "Turkmenistan", "Ukraine", "Uzbekistan"), convYear = 1992)
	splitting_states = rbind(splitting_states, data.table(oldC="Ethiopia PDR", newC = c("Ethiopia", "Eritrea"), convYear = 1993))
	splitting_states = rbind(splitting_states, data.table(oldC="Belgium-Luxembourg", newC = c("Belgium", "Luxembourg"), convYear = 2000))
	splitting_states = rbind(splitting_states, data.table(oldC="Yugoslav SFR", newC = c("Bosnia and Herzegovina", "Slovenia", "Croatia", "Serbia","Montenegro","The former Yugoslav Republic of Macedonia"), convYear = 1992))
	splitting_states = rbind(splitting_states, data.table(oldC="Czechoslovakia", newC = c("Czech Republic", "Slovakia"), convYear = 1994))
	splitting_states = rbind(splitting_states, data.table(oldC="Sudan (former)", newC = c('Sudan', 'South Sudan', 'Somalia'), convYear = 2011))
	
	setkey(splitting_states, newC, convYear)
	setkey(wbCountryDt, Country, Year)
	splitting_states = wbCountryDt[splitting_states][, list(newC=Country, convYear=Year, pop_ratio=population/sum(population)), by=oldC]
	
	Dt = Dt[!((Country %in% splitting_states[oldC=="USSR"]$newC & Year < 1992) | (Country %in% splitting_states[oldC=="Ethiopia PDR"]$newC & Year < 1993)|
						(Country %in% splitting_states[oldC=="Belgium-Luxembourg"]$newC & Year < 2000)	|(Country %in% splitting_states[oldC=="Yugoslav SFR"]$newC & Year < 1992)|
						(Country %in% splitting_states[oldC=="Czechoslovakia"]$newC & Year < 1994)|(Country %in% splitting_states[oldC=="Sudan (former)"]$newC & Year < 2011))] 
	
	adjustValue = function (x, y) { x * y}
	convert_com_bal = function (newC, factor, dt, colNames) {
		dt$Country = newC
		adjusted = dt[, lapply(.SD, adjustValue, y=factor), .SDcols = colNames]
		for (i in length(colNames):1) {
			dt[,colNames[i] := NULL]
		}
		cbind(dt, adjusted)
	}
	
	for (i in 1:length(splitting_states$newC)) {
		ss = splitting_states[i]
		print(ss$newC)
		Dt = rbind(convert_com_bal(ss$newC, ss$pop_ratio, Dt[Country == ss$oldC & Year <= ss$convYear], colsToChange), Dt, use.names=TRUE)
	}
	
	Dt = Dt[!((Country == "USSR" & Year < 1992) | (Country == "Ethiopia PDR" & Year < 1993) |(Country =="Belgium-Luxembourg"& Year < 2000)|
						(Country =="Yugoslav SFR" & Year < 1992)|(Country =="Serbia and Montenegro"& Year < 2006)|	(Country =="Czechoslovakia" & Year < 1994)|
						(Country =="Sudan (former)"& Year < 2011))] 
	Dt
}


getHistoricDataset  = function(commodityDt, countryDt) {
	
	h_dt = commodityDt[, list(Country, Year, Item, plumDemandItem, dryMatter, fcr,
					net_import=net_import, 
					food=food,
					feed=feed,
					other=other)]
	
	h_dt[, dm_food := food*dryMatter] ##TODO this doesn't work now because processed goods have been converted to their raw equivelents
	
	# animal product feed conversion adjustment
	h_dt[!is.na(fcr), c('food', 'net_import','feed','other') := list(food * fcr, net_import * fcr, feed * fcr, other *fcr)]
	
	h_dt2 = h_dt[, list(food=sum(food, na.rm=TRUE),
					net_import=sum(net_import, na.rm=TRUE), 
					feed=sum(feed, na.rm=TRUE),
					other=sum(other, na.rm=TRUE),
					avg_fcr_dm_adj=sum(dm_food, na.rm=TRUE)/sum(food, na.rm=TRUE) #TODO factor to convert from wet to dry, and also from feed equivalent quantities back to real quantities
			), by=list(Item=plumDemandItem, Country, Year)]
	
	h_dt2 = merge(h_dt2, countryDt[, list(Country, Year, gdp,population)], by=c('Country', 'Year'))
	h_dt2[, c('gdpPc', 'cpc') := list(gdp/population, food/population)]
}

getBaseConsumption  = function(year,consumpDt, gdpDt) {
	dt = consumpDt[Year == year, list(Item, Country, baseYear=Year, 
					avg_fcr_dm_adj,
					baseCpc=cpc,
					basePop=population, 
					baseGdpPc=gdp/population)]
	dt
}

getCommodityCurves = function(Dt) {
	dt = Dt[cpc > 0 & !is.na(cpc) & Country != "World"]
	
	curves=data.table(Item="test", a=NA, b=NA, c=NA, r2=NA, p=NA, fitType=NA)
	
	for (itemName in unique(dt$Item)) {
		dt1 = dt[Item==itemName]
		if (length(dt1$gdpPc) > 10) {
			print (itemName)
			#	fit1 = lm(cpc ~ log(gdpPc) - 1, data=dt1, weights=dt1$population)
			#	curves = rbind(curves, data.table(Item=itemName, a=0, b=fit1$coefficients[1], c=0,
			fit1 = lm(cpc ~ log(gdpPc), data=dt1, weights=dt1$population)
			curves = rbind(curves, data.table(Item=itemName, a=fit1$coefficients[1], b=fit1$coefficients[2], c=0,
							r2=summary(fit1)$adj.r.squared, p=anova(fit1)$'Pr(>F)'[1], fitType="loglinear"))
			
			#	fit2 = lm(cpc ~ log(gdpPc) + I(log(gdpPc)^2) - 1, dt1, weights=dt1$population)
			#	curves = rbind(curves, data.table(Item=itemName, a=0, b=fit2$coefficients[1], c=fit2$coefficients[2],
			#fit2 = lm(cpc ~ log(gdpPc) + I(log(gdpPc)^2), dt1, weights=dt1$population)
			#curves = rbind(curves, data.table(Item=itemName, a=fit2$coefficients[1], b=fit2$coefficients[2], c=fit2$coefficients[3],
			#				r2=summary(fit2)$adj.r.squared, p=anova(fit2)$'Pr(>F)'[1], fitType="kuznets"))
			
			#y = dt1[,cpc]
			#x = dt1[,gdpPc]
			
			#try (fit3 <- nls(y ~ SSlogis(x, a, b, c), weights=dt1$population))	# nls(y ~ a / (1 + exp((b-x)/c)), start = list(a=1, b= 100, c = 100))
			#curves = rbind(curves, data.table(Item=itemName, a=summary(fit3)$coef[1], b=summary(fit3)$coef[2], c=summary(fit3)$coef[3], r2=NA, p=NA, fitType="logistic"))
		}
	}
	curves[!is.na(b)]
}

getProjectedConsumption = function(fitType, gdpPc, a, b, c) {
	if (fitType == "logistic") {
		v = SSlogis(gdpPc, a, b, c)
	}
	else {
		v = a + b*log(gdpPc) + c*log(gdpPc)^2
	}
	as.numeric(v)
}

# Join ssp and baseConsumption
getProjectAllConsumption  = function(consumptionDt,sspDt, comCurvesDt) {
	setkey(consumptionDt, Country)
	setkey(sspDt, Country, Model, Scenario)
	dt=consumptionDt[sspDt, allow.cartesian=TRUE]	
	dt=dt[!is.na(Item) & baseCpc > 0 & Country != "World"]    # baseConsumption[Country %in% unique(dt[is.na(Item),Country])] # some countries are not in com_bal
	
	dt[, gdpPc:=gdp/population]
	setkey(dt, Item)
	setkey(comCurvesDt, Item, fitType)
	dt = dt[comCurvesDt, allow.cartesian=TRUE]
	dt=dt[!is.na(a) & !is.na(population) & !is.na(baseGdpPc)]     # unique(dt[is.na(i),Item]) # some Items not in com_bal + Grenada GRN does not have a projection in some SSP models
	
	z = 0 # z=0 is no dietary convergence    log(0.9)/2 # choose closing speed so that 10% reduction (still 90% gap) after increase to 2 times size, i.e. 100% increase 
	
	dt[,c('baseExpCpc', 'newExpCpc', 'gapRatio') := list(
					#if (fitType == "logistic") SSlogis(baseGdpPc, a, b, c) else a + b*log(baseGdpPc) + c*log(baseGdpPc)^2,					
					getProjectedConsumption(fitType, baseGdpPc, a, b, c),  # (a+b*log(baseGdpPc)+c*log(baseGdpPc)^2),
					getProjectedConsumption(fitType, gdpPc, a, b, c),  # (a+b*log(gdpPc)+c*log(gdpPc)^2),
					exp((gdpPc-baseGdpPc)/baseGdpPc*z) ), by=list(Model, Scenario, Year, Country, Item, fitType)]
	
	dt[gapRatio>1, gapRatio:=1] # if gdp drops
	
	dt[,projectedCpc:= newExpCpc/baseExpCpc*(baseCpc*gapRatio+baseExpCpc*(1-gapRatio))]   #(newExpCpc+(baseCpc-baseExpCpc)*gapRatio)]
	dt[projectedCpc<0, projectedCpc:=0]
	dt
}

extractionValues = function(dt, dtProcessed){

	# Get commodities produced from processing
	dt1= merge(dtProcessed[Year==2010,list(prod=sum(prod)),by=c('Country','processed')],
			dt[Year==2010,list(Country,processed=Item,rawAmountProcessed = processing)],
			by=c('Country','processed'))
	
	dt1 = dt1[prod > 0 & rawAmountProcessed > 0]
	dt1[,extractionRate:=prod/rawAmountProcessed]

	
	extractionValues = dt1[Country=='World',list(processed,extractionRate)]
	
	extractionValues
}
	
processData = function(minYear=1960, year=2010){
	print(paste("Base year is ", year))
	
	com_bal = readCommoditiyBalance()
	food_supply = readFoodSupply()
	cropProduction = readCropProd()
	# Original FAO data for commodities of interest
	cb_fs = merge(com_bal, food_supply, by=c("Country", "Year", "Item"), all.x=TRUE)
	cb_fs = merge(cb_fs, itemDetails[, list(Item, plumCropItem, plumDemandItem, fcr, dryMatter)], by="Item")
	
	
	cb_fs_sugar = cb_fs[Item %in% c("Sugar cane", "Sugar beet"), lapply(.SD, sum, na.rm=TRUE), by=list(Country, Year), .SDcols=c("supply", "export", "food", "feed", "import", "other", "processing", "prod", "dryMatter", "energy_kcal_pc", "stockvar")]
	cb_fs_sugar[, c('Item', 'dryMatter', 'fcr', 'plumCropItem', 'plumDemandItem') := list("Sugar crops", dryMatter/2, NA, 'Sugar', 'Sugar')] # find mean of DM from sum	
	cb_fs = rbind(cb_fs[!Item %in% c("Sugar cane", "Sugar beet"), list(Country, Year, Item, supply, export, food, feed, import,other, processing, prod, dryMatter, fcr, energy_kcal_pc, stockvar, plumCropItem, plumDemandItem)], cb_fs_sugar)
	
	itemMapping = fread("~/Documents/R_Workspace/landalloc/itemsProcessingMap.csv")
	processedDt = merge(cb_fs[Item %in% itemMapping$resultant], itemMapping[, list(processed, Item=resultant)], by="Item", all.x=TRUE)	
	extractionRates = extractionValues(cb_fs,processedDt)
	itemMapping = merge(itemMapping, extractionRates[,list(processed,extractionRate)],by='processed' )
	
	#below converts processed goods into their raw equivalent values i.e. now columns for soybean oil are in soybean equiv. terms 
	cb_fs3=merge(cb_fs,itemMapping[,list(Item=resultant,extractionRate)],by='Item',all=TRUE)
	colsToAdj = c("supply", "export", "food", "feed", "import", "other", "processing", "stockvar", "dryMatter")
	cb_fs3[!is.na(extractionRate), (colsToAdj) := lapply(.SD, function(x) x/extractionRate), by=list(Country, Year, Item), .SDcols=colsToAdj]
	cb_fs3[, net_import := import-export]
	
	#want to zero production of processed final goods as their production comes from the production of the raw good anyway 
	cb_fs3[Item %in% itemMapping$resultant, prod := 0]
	
	histDt = getHistoricDataset(cb_fs3,country_data)[Year>=minYear]
	ssp=getSsp(country_data)
	com_curves = getCommodityCurves(histDt)
	baseConsumption = getBaseConsumption(year,histDt,ssp)
	projectedConsumption = getProjectAllConsumption(baseConsumption,ssp[Year >= year],com_curves)
	livestockNumbers = readLivestockNumbers()
	
	return(list(ssp=ssp,histDt=histDt, baseConsumption=baseConsumption, com_curves=com_curves, country_data=country_data,
					projectedConsumption=projectedConsumption, cb_fs=cb_fs3, livestockNumbers=livestockNumbers))
}

processedFull = processData()
